﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using System.IO.Ports;
using System;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

public class GameController : MonoBehaviour
{

    public static SerialPort myPort { get; set; }

    [SerializeField] Text txtSerialData;
    [SerializeField] Dropdown ddPorts;
    [SerializeField] Button btnRefresh;

    public float speed;
    public float? calibratedDirection;
    public float direction;
    public bool reset;
    public bool breakOn;

    public static bool isConnected = false;

    // Start is called before the first frame update
    void Start()
    {
        if (myPort == null)
        {
            Load();
        }

    }

    // Update is called once per frame
    void Update()
    {
        if (isConnected)
        {
            try
            {
                string readout = myPort.ReadLine();
                txtSerialData.text = readout;
                string[] values = readout.Split(';');
                breakOn = values[0] == "1";
                if (calibratedDirection == null) calibratedDirection = Convert.ToSingle(values[1]);
                direction = Convert.ToSingle(values[1]);
                speed = Math.Min(2, Convert.ToSingle(values[2].Replace('.',',')))/2;
                reset = !(values[3] == "1");

                txtSerialData.text = $"break: {breakOn.ToString()}   direction: {direction.ToString()}   speed: {speed.ToString()}   reset: {breakOn.ToString()}";
            }
            catch (System.Exception ex)
            {
                //Debug.LogWarning(ex.Message);
            }
        }

    }

    public void RefreshPorts()
    {
        List<string> ports = new List<string>();
        foreach (string port in SerialPort.GetPortNames())
        {
            ports.Add(port);
        }
        ddPorts.ClearOptions();
        ddPorts.captionText.fontSize = 25;
        ddPorts.AddOptions(ports);
    }

    public void Connect()
    {
        Initialize(ddPorts.captionText.text);
    }
    public void Disconnect()
    {
        try
        {
            myPort.Close();
            isConnected = true;
        }
        catch
        {

        }
    }

    private void Initialize(string port)
    {        
        //print(")connecting
        myPort = new SerialPort(port, 115200);
        try
        {
            myPort.Open();
            myPort.ReadTimeout = 10;
            isConnected = true;
        }
        catch
        {

        }

    }

    public void Save()
    {
        BinaryFormatter bf = new BinaryFormatter();
        FileStream file = File.Create(Application.persistentDataPath + "/gamesave.save");
        bf.Serialize(file, ddPorts.captionText.text);
        file.Close();
        Debug.Log("Saved");
    }
    public void Load()
    {
        // 1
        if (File.Exists(Application.persistentDataPath + "/gamesave.save"))
        {
            // 2
            BinaryFormatter bf = new BinaryFormatter();
            FileStream file = File.Open(Application.persistentDataPath + "/gamesave.save", FileMode.Open);
            string text = (string)bf.Deserialize(file);
            file.Close();
            Initialize(text);
        }
        else
        {
            Debug.Log("No game saved!");
        }
    }
}
